@@ -146,6 +146,10 @@ static void
 illegal_instruction_handler (int num)
 {
   if (in_try_block) {
+    sigset_t set;
+    sigemptyset (&set);
+    sigaddset (&set, SIGILL);
+    sigprocmask (SIG_UNBLOCK, &set, NULL);
     longjmp (jump_env, 1);
   } else {
     abort ();
