@@ -32,49 +32,24 @@
 #include <emmintrin.h>
 #include <liboil/liboilcolorspace.h>
 
-/* Work around non-aligned stack frames (which causes the intristics to crash
- * by making sure the stack frame is always aligned
- */
-#if defined(__i386__)
-#define OIL_DEFINE_IMPL_FULL_WRAPPER(sse_name, name, flags, ret, ...)	\
-    ret sse_name(__VA_ARGS__) __attribute__((used));                    \
-    ret sse_name ## _wrap (__VA_ARGS__) {                               \
-        OIL_SSE_WRAPPER_CALL(sse_name);                                 \
-    }									\
-    OIL_DEFINE_IMPL_FULL(sse_name ## _wrap, name, flags);
-
-#define OIL_SSE_WRAPPER_CALL(name)                                      \
-    asm volatile(                                                       \
-            "\n\t"                                                      \
-            "subl $0x10,%%esp\n\t"                                      \
-            "andl $0xfffffff0,%%esp\n\t"                                \
-                                                                        \
-            "movdqu 8(%%ebp),%%xmm0\n\t"                                \
-            "movdqa %%xmm0,(%%esp)\n\t"                                 \
-                                                                        \
-            "call " #name "\n\t"                                        \
-            "movl %%ebp,%%esp\n\t"                                      \
-            : :                                                         \
-            : "eax","ecx","edx","xmm0")
-
-#elif defined(__amd64__)
-
-/* Needed because we call *_wrap. Should get optimized away anyway */ 
-
-#define OIL_DEFINE_IMPL_FULL_WRAPPER(sse_name, name, flags, ret, ...)	\
-    OIL_DEFINE_IMPL_FULL(sse_name, name, flags);
-
-#else
-#error Can't use sse on !i386 and !amd64
-#endif
+#include "sse_wrapper.h"
 
 
-static const __m128i c_sse_16xff =
-        {0xffffffffffffffffULL, 0xffffffffffffffffULL};
-static const __m128i c_sse_8x0080 =
-        {0x0080008000800080ULL, 0x0080008000800080ULL};
 
-#define MC(x) (c_sse_##x)
+union m128_int {
+  __m128i m128;
+  uint64_t ull[2];
+};
+
+static const struct _SSEData {
+  union m128_int sse_16xff;
+  union m128_int sse_8x0080;
+} c = {
+    .sse_16xff.ull =	{0xffffffffffffffffULL, 0xffffffffffffffffULL},
+    .sse_8x0080.ull =	{0x0080008000800080ULL, 0x0080008000800080ULL},
+};
+
+#define MC(x) (c.sse_##x.m128)
 
 /* non-SSE2 compositing support */
 #define COMPOSITE_OVER(d,s,m) ((d) + (s) - oil_muldiv_255((d),(m)))
@@ -222,11 +197,8 @@ composite_in_argb_sse (uint32_t *dest, const uint32_t *src, const uint8_t *mask,
 	COMPOSITE_IN(oil_argb_B(s), m));
   }
 }
-
 OIL_DEFINE_IMPL_FULL_WRAPPER (composite_in_argb_sse, composite_in_argb,
-    OIL_IMPL_FLAG_SSE2,
-    static void,
-    uint32_t *dest, const uint32_t *src, const uint8_t *mask, int n);
+    OIL_IMPL_FLAG_SSE2);
 
 static void
 composite_in_argb_const_src_sse (uint32_t *dest, const uint32_t *src,
@@ -262,11 +234,8 @@ composite_in_argb_const_src_sse (uint32_t *dest, const uint32_t *src,
 	COMPOSITE_IN(oil_argb_B(*src), m));
   }
 }
-
 OIL_DEFINE_IMPL_FULL_WRAPPER (composite_in_argb_const_src_sse,
-      composite_in_argb_const_src, OIL_IMPL_FLAG_SSE2,
-      static void,
-      uint32_t *dest, const uint32_t *src, const uint8_t *mask, int n);
+    composite_in_argb_const_src, OIL_IMPL_FLAG_SSE2);
 
 static void
 composite_in_argb_const_mask_sse (uint32_t *dest, const uint32_t *src,
@@ -303,9 +272,7 @@ composite_in_argb_const_mask_sse (uint32_t *dest, const uint32_t *src,
   }
 }
 OIL_DEFINE_IMPL_FULL_WRAPPER (composite_in_argb_const_mask_sse,
-    composite_in_argb_const_mask, OIL_IMPL_FLAG_SSE2,
-    static void, 
-    uint32_t *dest, const uint32_t *src, const uint8_t *mask, int n);
+    composite_in_argb_const_mask, OIL_IMPL_FLAG_SSE2);
 
 static void
 composite_over_argb_sse (uint32_t *dest, const uint32_t *src, int n)
@@ -339,7 +306,7 @@ composite_over_argb_sse (uint32_t *dest, const uint32_t *src, int n)
     *dest++ = d;
   }
 }
-OIL_DEFINE_IMPL_FULL (composite_over_argb_sse, composite_over_argb,
+OIL_DEFINE_IMPL_FULL_WRAPPER (composite_over_argb_sse, composite_over_argb,
     OIL_IMPL_FLAG_SSE2);
 
 static void
@@ -376,11 +343,8 @@ composite_over_argb_const_src_sse (uint32_t *dest, const uint32_t *src, int n)
     *dest++ = d;
   }
 }
-
 OIL_DEFINE_IMPL_FULL_WRAPPER (composite_over_argb_const_src_sse,
-    composite_over_argb_const_src, OIL_IMPL_FLAG_SSE2,
-    static void, 
-    uint32_t *dest, const uint32_t *src, int n);
+    composite_over_argb_const_src, OIL_IMPL_FLAG_SSE2);
 
 static void
 composite_in_over_argb_sse (uint32_t *dest, const uint32_t *src,
@@ -432,7 +396,7 @@ composite_in_over_argb_sse (uint32_t *dest, const uint32_t *src,
     *dest++ = d;
   }
 }
-OIL_DEFINE_IMPL_FULL (composite_in_over_argb_sse, composite_in_over_argb,
+OIL_DEFINE_IMPL_FULL_WRAPPER (composite_in_over_argb_sse, composite_in_over_argb,
     OIL_IMPL_FLAG_SSE2);
 
 static void
@@ -488,9 +452,7 @@ composite_in_over_argb_const_src_sse (uint32_t *dest, const uint32_t *src,
   }
 }
 OIL_DEFINE_IMPL_FULL_WRAPPER (composite_in_over_argb_const_src_sse,
-    composite_in_over_argb_const_src, OIL_IMPL_FLAG_SSE2,
-    static void, 
-    uint32_t *dest, const uint32_t *src, const uint8_t *mask, int n);
+    composite_in_over_argb_const_src, OIL_IMPL_FLAG_SSE2);
 
 static void
 composite_in_over_argb_const_mask_sse (uint32_t *dest, const uint32_t *src,
@@ -544,11 +506,8 @@ composite_in_over_argb_const_mask_sse (uint32_t *dest, const uint32_t *src,
     *dest++ = d;
   }
 }
-
 OIL_DEFINE_IMPL_FULL_WRAPPER (composite_in_over_argb_const_mask_sse,
-    composite_in_over_argb_const_mask, OIL_IMPL_FLAG_SSE2,
-    static void, 
-    uint32_t *dest, const uint32_t *src, const uint8_t *mask, int n);
+    composite_in_over_argb_const_mask, OIL_IMPL_FLAG_SSE2);
 
 static void
 composite_over_u8_sse (uint8_t *dest, const uint8_t *src, int n)
@@ -574,5 +533,6 @@ composite_over_u8_sse (uint8_t *dest, const uint8_t *src, int n)
     dest++;
   }
 }
-OIL_DEFINE_IMPL_FULL (composite_over_u8_sse, composite_over_u8,
+OIL_DEFINE_IMPL_FULL_WRAPPER (composite_over_u8_sse, composite_over_u8,
     OIL_IMPL_FLAG_SSE2);
+
