@@ -102,7 +102,6 @@ sub check
 	"call", 
 	"cld", 
 	"cltd", 
-	"cmovg", 
 	"cmp", 
 	"cmpb", 
 	"cmpl", 
@@ -119,6 +118,7 @@ sub check
 	"fcomp",
 	"fcompp",
 	"fdiv",
+	"fdivr",
 	"fdivrl", 
 	"fdivrs",
 	"fdivs",
@@ -164,6 +164,7 @@ sub check
 	"fucomp", 
 	"fucompp", 
 	"fxch", 
+	"idiv",
 	"imul", 
 	"imulb",
 	"inc", 
@@ -191,6 +192,7 @@ sub check
 	"movsbw", 
 	"movswl", 
 	"movsww", 
+	"movw",
 	"movzbl", 
 	"movzbw", 
 	"movzwl", 
@@ -203,6 +205,7 @@ sub check
 	"pop", 
 	"push", 
 	"pushl", 
+	"rep",
 	"repz", 
 	"ret", 
 	"rol", 
@@ -210,6 +213,7 @@ sub check
 	"sahf", 
 	"sar", 
 	"sarl", 
+	"setl",
 	"shl", 
 	"shll",
 	"shr", 
@@ -227,7 +231,11 @@ sub check
 	"cmova",
 	"cmovae",
 	"cmovb",
+	"cmovg", 
+	"cmovge", 
 	"cmovl",
+	"cmovle", 
+	"cmovbe",
 	"fcmovbe",
 	"fcmovnbe",
 );
